#!/usr/bin/env python

"""
****************************************************************************
 install.py, Linux install script for ConvertAll

 Copyright (C) 2004, Douglas W. Bell

 This is free software; you can redistribute it and/or modify it under the
 terms of the GNU General Public License, Version 2.  This program is
 distributed in the hope that it will be useful, but WITTHOUT ANY WARRANTY.
*****************************************************************************
"""

import sys, os.path, getopt, shutil, compileall, py_compile, glob, \
       re, distutils.sysconfig

prefixDir = '/usr/local'
progName = 'convertall'
testXML = 0
siteCustomReqd = 0

def usage(exitCode=2):
    """Display usage info and exit"""
    global prefixDir
    print 'Usage:'
    print '    python install.py [-h] [-p dir] [-x]'
    print 'where:'
    print '    -h         display this help message'
    print '    -p dir     install prefix [default: %s]' % prefixDir
    print '    -x         skip all dependency checks (risky)'
    sys.exit(exitCode)


def cmpVersions(versionStr, reqdTuple):
    """Return 1 if point-sep values in versionStr are >= reqdTuple"""
    match = re.search(r'[0-9\.]+', versionStr)
    if not match:
        return 0
    versionStr = match.group()
    versionList = [int(val) for val in versionStr.split('.') if val]
    reqdList = list(reqdTuple)
    while len(versionList) < len(reqdList):
        versionList.append(0)
    while len(reqdList) < len(versionList):
        reqdList.append(0)
    if cmp(versionList, reqdList) != -1:
        return 1
    return 0

def copyDir(srcDir, dstDir):
    """Copy all regular files from srcDir to dstDir,
       dstDir is created if necessary"""
    if not os.path.isdir(dstDir):
        os.makedirs(dstDir)
    names = os.listdir(srcDir)
    try:
        for name in names:
            shutil.copy2(os.path.join(srcDir, name), os.path.join(dstDir, name))
    except IOError, e:
        if str(e).find('Permission denied') >= 0:
            print 'Error - must be root to install files'
            cleanSource()
            sys.exit(4)
        raise

def createWrapper(execDir, execName):
    """Create a wrapper executable file for a python script in execDir
       named execName"""
    text = '#!/bin/sh\n\nexec %s %s/%s.py $*' % (sys.executable, execDir, \
                                                 execName)
    f = open(execName, 'w')
    f.write(text)
    f.close()
    os.chmod(execName, 0755)

def replaceLine(path, origLineStart, newLine):
    """Searches for origLineStart in file named path, 
       replaces all ocurrances with newLine and re-writes file"""
    f = open(path, 'r')
    lines = f.readlines()
    f.close()
    f = open(path, 'w')
    for line in lines:
        if line.startswith(origLineStart):
            f.write(newLine)
        else:
            f.write(line)
    f.close()

def createSiteCustomize():
    """Append the contents of my sitecustomize.py file to the one in
       site-packages if necessary"""
    f = open('source/sitecustomize.py', 'r')
    lines = f.readlines()
    f.close()
    siteDir = distutils.sysconfig.get_python_lib()
    sitePath = os.path.join(siteDir, 'sitecustomize.py')
    if os.access(sitePath, os.R_OK):
        f = open(sitePath, 'r')
        origLines = f.readlines()
        f.close()
        if lines[-1] in origLines:
            print '  Sitecustomize already in place.'
            return    # function already saved
    print '  Writing sitecustomize file'
    try:
        f = open(sitePath, 'a')
        f.writelines(lines)
        f.close()
        py_compile.compile(sitePath)
    except IOError, e:
        if str(e).find('Permission denied') >= 0:
            print '    Warning - must be root to modify sitecustomize file'
            print '    Skipping sitecustomize change'
            print '    Unicode may not function properly'
            return
        raise

def cleanSource():
    """Remove any temporary files added to untarred dirs"""
    for name in glob.glob(os.path.join('source', '*.py[co]')):
        os.remove(name)
    global progName
    if os.path.isfile(progName):
        os.remove(progName)

def removeDir(dir):
    """Remove dir and all files at path, ignore errors"""
    try:
        shutil.rmtree(dir, 1)
    except:    # shouldn't be needed with ignore error param, but
        pass   # some python versions have a bug

def main():
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hp:x')
    except getopt.GetoptError:
        usage(2)
    global prefixDir
    global progName
    depCheck = 1
    for opt, val in opts:
        if opt == '-h':
            usage(0)
        elif opt == '-p':
            prefixDir = val
        elif opt == '-x':
            depCheck = 0
    if not os.path.isdir('source'):
        print 'Error - %s source files not found' % progName
        print 'The directory containing "install.py" must be current'
        sys.exit(4)
    if depCheck:
        print 'Checking dependencies...'
        try:
            pyVersion = sys.version_info[:3]
        except AttributeError:
            print '  Python Version 1.x -> Sorry, 2.1 or higher is required'
            sys.exit(3)
        pyVersion = '.'.join([str(num) for num in pyVersion])
        if cmpVersions(pyVersion, (2, 1)):
            print '  Python Version %s -> OK' % pyVersion
        else:
            print '  Python Version %s -> Sorry, 2.1 or higher is required' \
                  % pyVersion
            sys.exit(3)
        try:
            from qt import qVersion, PYQT_VERSION
        except:
            print '  Sorry, PyQt must be installed'
            sys.exit(3)
        qtVersion = qVersion()
        if cmpVersions(qtVersion, (2, 3)):
            print '  Qt Version %s -> OK' % qtVersion
        else:
            print '  Qt Version %s -> Sorry, 2.3 or higher is required' \
                  % qtVersion
            sys.exit(3)
        try:
            from qt import PYQT_VERSION_STR
            pyqtVersion = PYQT_VERSION_STR
        except:
            pyqtVersion = PYQT_VERSION
        if cmpVersions(pyqtVersion, (2, 4)):
            print '  PyQt Version %s -> OK' % pyqtVersion
        else:
            print '  PyQt Version %s -> Sorry, 2.4 or higher is required' \
                  % pyqtVersion
            sys.exit(3)
        global testXML
        if testXML:
            try:
                import xml.sax
                handler = xml.sax.ContentHandler()
                xml.sax.parseString('<XML>test</XML>', handler)
            except:
                print '  XML Parser -> Sorry, the expat library or '\
                      'PyXML package is required'
                sys.exit(3)
            print '  XML Parser -> OK'
    compileall.compile_dir('source', ddir=os.path.join(prefixDir, 'source'))
    print 'Installing files...'
    pyDir = os.path.join(prefixDir, 'lib', progName)
    print '  Copying python files to %s' % pyDir
    removeDir(pyDir)         # remove old?
    copyDir('source', pyDir)
    if os.path.isdir('doc'):
        docDir = os.path.join(prefixDir, 'share', 'doc', progName)
        print '  Copying documentation files to %s' % docDir
        removeDir(docDir)   # remove old?
        copyDir('doc', docDir)
        # update help file location in main python script
        replaceLine(os.path.join(pyDir, '%s.py' % progName), \
                    'helpFilePath = None', \
                    'helpFilePath = \'%s\'   # modified by install script\n' \
                    % docDir)
    if os.path.isdir('data'):
        dataDir = os.path.join(prefixDir, 'share', progName)
        print '  Copying data files to %s' % dataDir
        removeDir(dataDir)   # remove old?
        copyDir('data', dataDir)
        # update data file location in main python script
        replaceLine(os.path.join(pyDir, '%s.py' % progName), \
                    'dataFilePath = None', \
                    'dataFilePath =  \'%s\'   # modified by install script\n' \
                    % dataDir)
    if os.path.isdir('icons'):
        iconDir = os.path.join(prefixDir, 'share', 'icons')
        print '  Copying icon files to %s' % iconDir
        copyDir('icons', iconDir)
    if siteCustomReqd:
        createSiteCustomize()
    createWrapper(pyDir, progName)
    binDir = os.path.join(prefixDir, 'bin')
    print '  Copying executable file "%s" to %s' % (progName, binDir)
    if not os.path.isdir(binDir):
        os.makedirs(binDir)
    shutil.copy2(progName, binDir)
    cleanSource()
    print 'Install complete.'


if __name__ == '__main__':
    main()
